from PyQt6 import QtCore, QtGui, QtWidgets
from PyQt6.QtCore import Qt

from functools import partial
import re


# ----------------------------------------------------------------------
class Dialog(QtWidgets.QWidget):

    # --------
    def __init__(self):
        super().__init__()
        self.setFixedSize(300, 400)

        layout = QtWidgets.QVBoxLayout()
        b = MenuButton(parent=self)
        layout.addWidget(b)
        layout.setAlignment(b, Qt.AlignmentFlag.AlignTop)
        self.setLayout(layout)

    # --------
    def keyPressEvent(self, event):
        mod, key = event.modifiers(), event.key()
        if (mod,key) == (Qt.KeyboardModifier.ControlModifier, Qt.Key.Key_Q):
            self.close()

        super().keyPressEvent(event)


# ----------------------------------------------------------------------
class MenuButton(QtWidgets.QToolButton):

    # --------
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setFixedSize(64, 64)
        self.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextOnly)
        self.setText('Menu')

        self.menu = Menu(parent=self)
        for i, s in enumerate(
            ['Anchorage', 'Biloxi', 'Boston', 'Helena', 'Honolulu', 'Las Vegas']
        ):
            self.menu.addAction(
                s,
                partial(self.menu_item, s)
            )

            if i == 2:
                submenu = Menu(title='Denver', parent=self)
                for t in ['Sub1', 'Sub2', 'Sub3']:
                    submenu.addAction(
                        t,
                        partial(self.menu_item, t)
                    )
                self.menu.addMenu(submenu)

        self.setMenu(self.menu)
        self.setPopupMode(
            QtWidgets.QToolButton.ToolButtonPopupMode.InstantPopup
        )

    # --------
    def menu_item(self, s):
        print(s)


# ------------------------------------------------------------------------------
class Menu(QtWidgets.QMenu):

    # --------
    def __init__(self, title=None, parent=None):
        super().__init__(title, parent)

    # --------
    def showEvent(self, event):
        super().showEvent(event)

        self.idx = 0

    # --------
    def keyPressEvent(self, event):
        mod, key = event.modifiers(), event.key()

        if (
            mod in (Qt.KeyboardModifier.NoModifier, Qt.KeyboardModifier.ShiftModifier) and
            key >= Qt.Key.Key_A and key <= Qt.Key.Key_Z
        ):
            a = [a.text() for a in self.actions()]
            starting_idx = self.idx
            idx = None
            while True:
                if re.match(chr(key), a[self.idx], re.IGNORECASE):
                    idx = self.idx

                self.idx = (self.idx + 1) % len(a)

                if idx is not None or self.idx == starting_idx:
                    break

            if idx is not None:
                a = self.actions()[idx]
                self.setActiveAction(a)

        super().keyPressEvent(event)
        


# ----------------------------------------------------------------------
# ----------------------------------------------------------------------
if __name__ == '__main__':

    app = QtWidgets.QApplication([])
    app.setStyle('Fusion')
    dlg = Dialog()
    dlg.show()

    exit(app.exec())
